;################################################################################
;#										#
;# libmio - multi i/o for ATMega644						#
;# main library									#
;# copyright (c) 2005-2010 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------
;port konfiguration for port B ... D
;-------------------------------------------------------------------------------
libmio_init:	ldi	r16,24
		ldi	XL,LOW(libmio_ram)
		ldi	XH,HIGH(libmio_ram)
libmio_init_r:	st	X+,const_0
		dec	r16
		brne	libmio_init_r

		ldi	XL,0x00			;port direction (+SPI)
		out	DDRB,XL
		ldi	XL,0x00
		out	PORTB,XL

		ldi	XL,0x00			;port direction
		out	DDRC,XL
		ldi	XL,0x00
		out	PORTC,XL

		ldi	XL,0x00			;set pin 3-7 as output
		out	DDRD,XL
		ldi	XL,0x00
		out	PORTD,XL

		clr	XL
		sbic	CONFPORT_0,CONFPIN_0
		ori	XL,0x01
		sbic	CONFPORT_1,CONFPIN_1
		ori	XL,0x02
		sbic	CONFPORT_2,CONFPIN_2
		ori	XL,0x04
		sts	libmio_sysconf,XL

;-------------------------------------------------------------------------------
;usart configuration (synchronous for keyboard)
;-------------------------------------------------------------------------------
libmio_t2i:	sts	UCSR0A,const_0
		ldi	XL,0x10
		sts	UCSR0B,XL
		ldi	XL,0x66
		sts	UCSR0C,XL

;-------------------------------------------------------------------------------
;i2c configuration (400kHz)
;-------------------------------------------------------------------------------
.if USE_I2C == 1
		ldi	XL,17		;speed
		sts	TWBR,XL		
		ldi	XL,0x00		;/1
		sts	TWSR,XL
.endif

		lds	XL,libmio_sysconf

.set	USE_CFG1 = 0

.if USE_LCD > 0
	.set USE_CFG1 = 1
.endif

.if USE_VGA > 0
	.set USE_CFG1 = 1
.endif

.if USE_CFG1 == 1
		sbrs	XL,1
.endif

.if USE_TV > 0 
		rjmp	init_tv
.else
		ret
.endif

.if USE_LCD > 0
	.if USE_VGA > 0
		sbrc	XL,0
	.endif
		jmp	init_lcd
.else
		ret
.endif

.if USE_VGA > 0
		jmp	init_vga
.else
		ret
.endif

;------------------------------------------------------------------------------
;wait for a key
;------------------------------------------------------------------------------
libmio_waitkey:	sts	libmio_lastkey,const_0
libmio_wkey1:	lds	r16,libmio_lastkey
		cpi	r16,0
		breq	libmio_wkey1
		ret

